<?php
/*
Plugin Name: WooCommerce MercadoPago Gateway
Plugin URI: http://woothemes.com/woocommerce
Description: Extends WooCommerce with a <a href="http://www.mercadopago.com" target="_blank">MercadoPago</a> gateway for WooCommerce.<br /> Email <a href="mailto:jmanuel@duplika.com">jmanuel@duplika.com</a> with any questions. This gateway requies a server with support for CURL.
Version: 1.1.2
Author: Mariano Pérez Rodriguez
Author URI: http://www.duplika.com

	Copyright 2011  Mariano Pérez Rodriguez (email : jmanuel@duplika.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as
	published by the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Required functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) )
	require_once( 'woo-includes/woo-functions.php' );

/**
 * Plugin updates
 */
woothemes_queue_update( plugin_basename( __FILE__ ), '4048a6591fd642a74767560b6ec22f0f', '18629' );

/**
 * Localisation
 **/
load_plugin_textdomain('wc_mercado', false, dirname( plugin_basename( __FILE__ ) ) . '/languages');

// Init MercadoPago Gateway after WooCommerce has loaded
add_action ('plugins_loaded', 'init_mercadopago_gateway', 0);

function init_mercadopago_gateway () {

	// If the WooCommerce payment gateway class is not available, do nothing
	if ( ! class_exists ( 'WC_Payment_Gateway' ) )  return;

	class WC_Gateway_Mercadopago extends WC_Payment_Gateway {

	    public function __construct () {
			$this->id          = 'mercadopago';
			$this->method_title = __('MercadoPago', 'wc_stripe');
			$this->icon        = plugins_url (basename (dirname (__FILE__)) . "/images/MP.png");
			$this->has_fields  = false;
			$this->purchaseurl = 'https://www.mercadopago.com/mla/buybutton';
			$this->sondaurl    = 'https://www.mercadopago.com/mla/sonda';

			// Load the form fields.
			$this->init_form_fields ();

			// Load the settings.
			$this->init_settings ();

			// Define user set variables
			$this->enabled     = $this->settings ['enabled'    ];
			$this->title       = $this->settings ['title'      ];
			$this->description = $this->settings ['description'];

			$this->account_id = $this->settings ['account_id'];
			$this->enc_code   = $this->settings ['enc_code'  ];
			$this->sonda_key  = $this->settings ['sonda_key'];

			$this->currency = $this->settings ['currency'];
			$this->debugmode       = $this->settings ['debugmode'      ];
			$this->debugmode_email = $this->settings ['debugmode_email'];
			$this->auto_submit = $this->settings ['auto_submit'];
			$this->name_template = 'Pedido ##1# en #2#';

			// Actions
			add_action ('valid_mercadopago_request', array (& $this, 'successful_request'        ));
			add_action ('woocommerce_receipt_mercadopago',             array (& $this, 'receipt_page'         ));
			add_action( 'woocommerce_update_options_payment_gateways', array( $this, 'process_admin_options' ) );
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

			// Payment listener/API hook WC_Gateway_Mercadopago
			add_action( 'woocommerce_api_wc_gateway_mercadopago', array(&$this, 'check_mercadopago_response') );

	    }  // End __construct ()


		/**
		* Initialise Gateway Settings Form Fields
		*/
		function init_form_fields () {
			$this->form_fields = array (
				'enabled' => array (
					'type'    => 'checkbox',
					'default' => 'yes',
					'title'   => __('Enable/Disable',     'wc_mercado'),
					'label'   => __('Enable MercadoPago', 'wc_mercado'),
				),
				'title' => array (
					'type'        => 'text',
					'default'     => __('MercadoPago',                                                  'wc_mercado'),
					'title'       => __('Title',                                                        'wc_mercado'),
					'description' => __('This controls the title which the user sees during checkout.', 'wc_mercado'),
				),
				'description' => array (
					'type'        => 'textarea',
					'default'     => __('Pay via MercadoPago secure processing.',                       'wc_mercado'),
					'title'       => __('Description',                                                  'wc_mercado'),
					'description' => __('This controls the title which the user sees during checkout.', 'wc_mercado'),
				),
				'account_id' => array (
					'type'        => 'text',
					'default'     => '',
					'title'       => __('MercadoPago account number',                                                             'wc_mercado'),
					'description' => __('Please enter your MercadoPago account number; this is needed in order to take payment!', 'wc_mercado'),
				),
				'enc_code' => array (
					'type'        => 'text',
					'default'     => '',
					'title'       => __('MercadoPago ENC Code',                                                             'wc_mercado'),
					'description' => __('Please enter your MercadoPago ENC Code; this is needed in order to take payment!', 'wc_mercado'),
				),
				'sonda_key' => array (
					'type'        => 'text',
					'default'     => '',
					'title'       => __('MercadoPago SONDA Key',                                                             'wc_mercado'),
					'description' => __('Please enter your MercadoPago SONDA Key; this is needed in order to take payment!', 'wc_mercado'),
				),
				'currency' => array (
					'type' => 'select',
					'default' => 'ARS',
					'title' => __('MercadoPago Currency'),
					'description' => __('Please enter your MercadoPago Currency; this is needed in order to take payment!', 'wc_mercado'),
					'options' => array (
						'ARS' => 'Pesos Argentinos',
						'REA' => 'Reales Brasileros',
					),
				),
				'debugmode' => array (
					'type'    => 'checkbox',
					'default' => 'yes',
					'title'   => __('Send Debug Emails',                                                                                                                          'wc_mercado'),
					'label'   => __('Enable Send Debug Emails. Receive one email containing the data sent to MercadoPago and one containing the data returned from MercadoPago.', 'wc_mercado'),
				),
				'debugmode_email' => array (
					'type'        => 'text',
					'default'     => get_bloginfo ('admin_email'),
					'title'       => __('Who gets the Debug emails',                                    'wc_mercado'),
					'description' => __('If Send Debug Emails is active, where do we send the emails?', 'wc_mercado'),
				),
				'auto_submit' => array (
					'type'    => 'checkbox',
					'default' => 'no',
					'title'   => __('Auto-submit payment form',                                     'wc_mercado'),
					'label'   => __('Auto-submit (JS-wise) the payment form going to MercadoPago.', 'wc_mercado'),
				),
			);

		}  // End init_form_fields ()


		/**
		 * Admin Panel Options
		 */
		public function admin_options() {
			?>
			<h3><?php _e ('MercadoPago', 'wc_mercado'); ?></h3>
			<p><?php _e ('MercadoPago works by sending the user to <a href = "http://www.mercadopago.com">MercadoPago</a> to enter their payment information. Instructions on how to set up the MP account settings can be found in the readme.txt.', 'wc_mercado'); ?></p>
			<table class = "form-table">
				<?php
				// Generate the HTML For the settings form.
				$this->generate_settings_html ();
				?>
			</table><!--/.form-table-->
			<?php
		}  // End admin_options ()


		/**
		 * There are no payment fields for MercadoPago, but we want to show the description if set.
		 */
		function payment_fields () {
			if ( $this->description )
				echo wpautop( wptexturize( $this->description ) );
		}  // End payment_fields ()


		/**
		 * Generate the MercadoPago button link
		 */
		public function generate_mercadopago_form ($order_id) {
			global $woocommerce;

			$order = new WC_Order ($order_id);

			$mercadopago_args = array (
				'acc_id' 			=> $this->account_id,
				'enc'    			=> $this->enc_code,
				'url_succesfull' 	=> add_query_arg( 'oid', $order_id, add_query_arg('res', 'succesfull', str_replace( 'https:', 'http:', add_query_arg('wc-api', 'WC_Gateway_Mercadopago', home_url( '/' ) ) ) ) ),
				'url_process'    	=> add_query_arg( 'oid', $order_id, add_query_arg('res', 'process', str_replace( 'https:', 'http:', add_query_arg('wc-api', 'WC_Gateway_Mercadopago', home_url( '/' ) ) ) ) ),
				'url_cancel'     	=> add_query_arg( 'oid', $order_id, add_query_arg('res', 'cancel', str_replace( 'https:', 'http:', add_query_arg('wc-api', 'WC_Gateway_Mercadopago', home_url( '/' ) ) ) ) ),
				'item_id'      		=> $order_id,
				'seller_op_id' 		=> $order_id,
				'name'         		=> build_product_name( $this->name_template, $order_id ),
				'currency'      	=> $this->currency,
				'price'         	=> $order->order_total,
				//'shipping_cost' 	=> $order->order_shipping, contained in order total
				'cart_name'    		=> $order->billing_first_name,
				'cart_surname' 		=> $order->billing_last_name,
				'cart_email'   		=> $order->billing_email,
			);

			// SEND THE DEBUG EMAIL
			if ($this->debugmode == 'yes' && isset ($this->debugmode_email)) {
				$message = '';
				foreach ($mercadopago_args as $key => $value) {
					$message .= $key . ' = ' . $value . "\r\n";
				}
				$message = 'Order ID: ' . $order_id . "\r\n" . "\r\n" . $message;
				mercadopago_debug_email ($this->debugmode_email, 'MercadoPago Debug. Sent Values Order ID: ' . $order_id, $message);
			}

			// Prepare the form
			$mercadopago_args_array = array ();
			foreach ($mercadopago_args as $key => $value) {
				$mercadopago_args_array [] = '<input type = "hidden"  name = "' . $key . '"  value = "' . $value . '" />';
			}

			if ($this->auto_submit == 'yes') {
				$woocommerce->add_inline_js('
				jQuery("body").block({
						message: "<img src=\"'.esc_url( $woocommerce->plugin_url() ).'/assets/images/ajax-loader.gif\" alt=\"Redirecting…\" style=\"float:left; margin-right: 10px;\" />'.__('Thank you for your order. We are now redirecting you to MercadoPago to make payment.', 'wc_amazon_payments').'",
						overlayCSS:
						{
							background: "#fff",
							opacity: 0.6
						},
						css: {
					        padding:        20,
					        textAlign:      "center",
					        color:          "#555",
					        border:         "3px solid #aaa",
					        backgroundColor:"#fff",
					        cursor:         "wait",
					        lineHeight:		"32px"
					    }
					});
				jQuery("#submit_mercadopago_payment_form").click();
				');
			}

			$form =
				'<form target = "_top"  action = "' . $this->purchaseurl . '"  method = "post">' . implode ('', $mercadopago_args_array) .
				'<input type = "submit"  class = "button-alt"  id = "submit_mercadopago_payment_form"  value = "' . __('Pay via MercadoPago', 'wc_mercado') . '" />' .
				'<a class = "button cancel"  href = "' . $order->get_cancel_order_url() . '">' . __('Cancel order &amp; restore cart', 'wc_mercado') . '</a>' .
				'</form>';

			return $form;
		}  // End generate_mercadopago_form ($order_id)


		/**
		 * Process the payment and return the result
		 */
		function process_payment ($order_id) {
			$order = new WC_Order ($order_id);

			return array (
				'result'   => 'success',
				'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(woocommerce_get_page_id('pay'))))
			);
		}  // End process_payment ($order_id)


		/**
		 * receipt_page
		 */
		function receipt_page ($order) {
			echo '<p>' . __('Thank you for your order, please click the button below to pay with MercadoPago.', 'wc_mercado') . '</p>';
			echo $this->generate_mercadopago_form ($order);
		}  // End receipt_page ($order)


	    /**
	     * Check for MercadoPago Response
	     */
	    function check_mercadopago_response() {

	    	if ( ! empty( $_REQUEST['res'] ) && ! empty( $_REQUEST['oid'] ) && in_array( $_REQUEST['res'], array('succesfull', 'process', 'cancel') ) ) {

		    	do_action ("valid_mercadopago_request", array (
		            'res' => $_REQUEST ['res'],
		            'oid' => $_REQUEST ['oid'],
		        ));

	    	}

	    }  // End check_mercadopago_response ()

	    /**
	     * Successful Payment!
	     */
	    function successful_request ($mercadopago_return_values) {
			global $woocommerce;

			$order_id  = $mercadopago_return_values ['oid'];
			$mp_result = $mercadopago_return_values ['res'];  // 'succesfull', 'process', 'cancel'

			// get order
			$order = new WC_Order ((int) $order_id);

			// Possible statuses: pending, failed, on-hold, processing, completed, refunded, cancelled
			if (in_array ($order->status, array ('pending', 'on-hold'))) {
				$response = null;

				// get status from MercadoPago
				$sonda_response = $this->probe_sonda ($order_id, $order);
				$response = $sonda_response ['operation']['status'];

				if (!empty ($response)) {
					// add the order number as a note
					$order->add_order_note (__('MercadoPago payment process returned. Order number: ', 'wc_mercado') . $order_id);
					switch ($response) {
						case 'A': $order->payment_complete(); break;
						case 'P': $order->update_status ('processing', __('Awaiting MercadoPago confirmation', 'wc_mercado')); break;
						case 'C': $order->update_status ('cancelled',  __('Received MercadoPago cancellation', 'wc_mercado')); break;
					}
				}
			}
			$woocommerce->cart->empty_cart();

			wp_redirect( $this->get_return_url( $order ) );
		    exit;

	    }  // End successful_request ($mercadopago_return_values)


	    /**
	     * Check response by means of SONDA
	     */
	    function probe_sonda ($order_id, $order) {
			// we need cURL! ;)
			if (!function_exists ('curl_init'))  return array();

			// set-up SONDA required parameters
			$sonda_args = array (
				'seller_op_id' => $order_id,
				'acc_id'       => $this->account_id,
				'sonda_key'    => $this->sonda_key,
			);

			// initialize SONDA response array
			$sonda_response_array = null;

	      	{ // begin cURL block
				// build post parameters
				$sonda_args_array = array ();
				foreach ($sonda_args as $key => $value) {
					$sonda_args_array [] = trim ($key) . '=' . trim ($value);
				}

				// create a new cURL resource for SONDA
				$sonda_ch = curl_init ($this->sondaurl);

				// set appropriate options
				curl_setopt ($sonda_ch, CURLOPT_MAXREDIRS, 7);
				curl_setopt ($sonda_ch, CURLOPT_POST, true);
				curl_setopt ($sonda_ch, CURLOPT_RETURNTRANSFER, true);
				curl_setopt ($sonda_ch, CURLOPT_POSTFIELDS, implode ('&', $sonda_args_array));
				curl_setopt ($sonda_ch, CURLOPT_HEADER, 0);

				// grab URL and pass it to cURL
				$sonda_response = curl_exec ($sonda_ch);
				// close cURL resource, and free up system resources
				curl_close ($sonda_ch);
				// try to parse XML response
				libxml_use_internal_errors (true);
				$sonda_response_xml = simplexml_load_string ($sonda_response);
				// get the array representation of the XML oject
				$sonda_response_array = object2array ($sonda_response_xml);
	      	} // end cURL block

			$isOK = true;

			// check presence and value of fields
			$isOK = $isOK && isset ($sonda_response_array ['message']) && ((string) $sonda_response_array ['message'] == 'OK');
			//
			$isOK = $isOK && isset ($sonda_response_array ['operation']);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['mp_op_id'         ]);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['seller_op_id'     ]) && ((int)    $sonda_response_array ['operation']['seller_op_id'     ] == (int)    $order_id);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['item_id'          ]) && ((int)    $sonda_response_array ['operation']['item_id'          ] == (int)    $order_id);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['acc_id'           ]) && ((string) $sonda_response_array ['operation']['acc_id'           ] == (string) $this->account_id);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['name'             ]) && ((string) $sonda_response_array ['operation']['name'             ] == (string) urlencode (build_product_name ($this->name_template, $order_id)));
			$isOK = $isOK && isset ($sonda_response_array ['operation']['price'            ]) && ((float)  $sonda_response_array ['operation']['price'            ] == (float)  $order->order_total);
			//$isOK = $isOK && isset ($sonda_response_array ['operation']['shipping_amount'  ]) && ((float)  $sonda_response_array ['operation']['shipping_amount'  ] == (float)  $order->order_shipping);
			$isOK = $isOK && isset ($sonda_response_array ['operation']['additional_amount']) && ((float)  $sonda_response_array ['operation']['additional_amount'] == (float)  0);
			//$isOK = $isOK && isset ($sonda_response_array ['operation']['total_amount'     ]) && ((float)  $sonda_response_array ['operation']['total_amount'     ] == (float)  ($order->order_total + $order->order_shipping));
			$isOK = $isOK && isset ($sonda_response_array ['operation']['extra_part'       ]) && (         $sonda_response_array ['operation']['extra_part'       ] ==          array ());
			//
			$isOK = $isOK && isset ($sonda_response_array ['operation']['status'            ]) && (in_array ((string) $sonda_response_array ['operation']['status'            ], array ('A', 'P', 'C')));
			$isOK = $isOK && isset ($sonda_response_array ['operation']['status_description']) && (in_array ((string) $sonda_response_array ['operation']['status_description'], array ('approved', 'cancelled', 'in_mediation', 'in_process', 'pending', 'refunded', 'rejected', 'uninitialized')));
			$isOK = $isOK && isset ($sonda_response_array ['operation']['payment_method'    ]) && (in_array ((string) $sonda_response_array ['operation']['payment_method'    ], array ('TCO', 'OFF')));

			return $isOK ? $sonda_response_array : array ();
		}  // End probe_sonda ($seller_op_id, $mp_op_id)

	} // Close woocommerce_mercadopago


	/**
	 * Generate product name according to template given the order_id
	 */
	function build_product_name ($template, $order_id) {
		return str_replace ('#1#', $order_id,
	       str_replace ('#2#', get_bloginfo ('name'),
	       str_replace ('#3#', get_bloginfo ('wpurl'), $template)));
	}


	/**
	 * Utility function to transform an onject into an array
	 */
	function object2array ($object) { return @json_decode (@json_encode ($object), 1); }


	/**
	 * Send a debug email
	 */
	function mercadopago_debug_email ($debugemail = null, $subject = null, $message = null) {
		// global $woocommerce;

		if ($debugemail == null) {
		  $debugemail = get_option ('admin_email');
		}
		if ($subject == null) {
		  $subject = 'MercadoPago Debug Email From:';
		}

		$subject .= ' ' . get_bloginfo ('name');

		woocommerce_mail ($debugemail, $subject, $message);

	}  // End mercadopago_debug_email

}  // Close init_mercadopago_gateway


/**
 * Add the gateway to WooCommerce
 */
function add_mercadopago_gateway ($methods)  { $methods [] = 'WC_Gateway_Mercadopago'; return $methods; }

add_filter ('woocommerce_payment_gateways', 'add_mercadopago_gateway');